/****************************************/
/* Cool Herders                         */
/* A demented muse by Brendan Wiese     */
/* http://starbow.org                   */
/* Programmed by Mike Brent             */
/* http://tursi.yiffco.com              */
/* sprite.c                             */
/****************************************/

/* these functions all assume the 256x256 textures described in bmp.c */
/* Sprites are tiles fixed at 32x32 pixels */

#include <stdio.h>
#include <malloc.h>
#include <kallisti/libk.h>
#include <kallisti/timer.h>
#include <kallisti/abi/fs.h>
#include <kallisti/abi/ta.h>
#include <kallisti/abi/spu.h>
#include <kallisti/abi/maple.h>
#include <kallisti/abi/video.h>
#include <kallisti/abi/thread.h>
#include <kallisti/abi/maple.h>
#include <kallisti/abi/sndsrv_abi.h>
#include <kallisti/vmu.h>
#include "sprite.h"
#include "cool.h"

void SortSprite(SPRITE *spr) {
	vertex_ot_t vert;
	poly_hdr_t poly;
	float u1, u2, v1, v2, off;
	int tmpX, tmpY;

	// calculate texture offsets
	// texture addresses are 8x8
	off=48.0/PAGESIZE;
	u1=((spr->tilenumber%5)*TILESIZE)/PAGESIZE;
	u2=u1+off;
	v1=((spr->tilenumber/5)*TILESIZE)/PAGESIZE;
	v2=v1+off;

	ta->poly_hdr_txr(&poly, spr->ta_style, TA_ARGB1555, PAGESIZE, PAGESIZE, spr->txr_addr, TA_NO_FILTER);
	ta->commit_poly_hdr(&poly);

	tmpX=spr->x-(TILESIZE-GRIDSIZE)/2;
	tmpY=spr->y-(TILESIZE-GRIDSIZE);

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = tmpX;
	vert.y = tmpY+TILESIZE;
	vert.z = spr->z;
	vert.u = u1;
	vert.v = v2;
	vert.a = spr->alpha;
	vert.r = 1.0f;
	vert.g = 1.0f;
	vert.b = 1.0f;
	vert.oa = vert.or = vert.og = vert.ob = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = tmpX;
	vert.y = tmpY;
	vert.z = spr->is3D ? spr->z+10.0 : spr->z;
	vert.u = u1;
	vert.v = v1;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = tmpX+TILESIZE;
	vert.y = tmpY+TILESIZE;
	vert.z = spr->z;
	vert.u = u2;
	vert.v = v2;
	ta->commit_vertex(&vert, sizeof(vert));

	vert.flags = TA_VERTEX_EOL;
	vert.x = tmpX+TILESIZE;
	vert.y = tmpY;
	vert.z = spr->is3D ? spr->z+10.0 : spr->z;
	vert.u = u2;
	vert.v = v1;
	ta->commit_vertex(&vert, sizeof(vert));
}

void SortScaledSprite(SPRITE *spr, float xScale, float yScale) {
	vertex_ot_t vert;
	poly_hdr_t poly;
	float u1, u2, v1, v2, off;
	int tmpX, tmpY;

	// calculate texture offsets
	// texture addresses are 8x8
	off=48.0/PAGESIZE;
	u1=((spr->tilenumber%5)*TILESIZE)/PAGESIZE;
	u2=u1+off;
	v1=((spr->tilenumber/5)*TILESIZE)/PAGESIZE;
	v2=v1+off;

	ta->poly_hdr_txr(&poly, TA_TRANSLUCENT, TA_ARGB1555, PAGESIZE, PAGESIZE, spr->txr_addr, TA_BILINEAR_FILTER);
	ta->commit_poly_hdr(&poly);

	tmpX=spr->x + (TILESIZE * xScale);
	tmpY=spr->y + (TILESIZE * yScale);

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = spr->x;
	vert.y = tmpY;
	vert.z = spr->z;
	vert.u = u1;
	vert.v = v2;
	vert.a = spr->alpha;
	vert.r = 1.0f;
	vert.g = 1.0f;
	vert.b = 1.0f;
	vert.oa = vert.or = vert.og = vert.ob = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = spr->x;
	vert.y = spr->y;
	vert.z = spr->is3D ? spr->z+10.0 : spr->z;
	vert.u = u1;
	vert.v = v1;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = tmpX;
	vert.y = tmpY;
	vert.z = spr->z;
	vert.u = u2;
	vert.v = v2;
	ta->commit_vertex(&vert, sizeof(vert));

	vert.flags = TA_VERTEX_EOL;
	vert.x = tmpX;
	vert.y = spr->y;
	vert.z = spr->is3D ? spr->z+10.0 : spr->z;
	vert.u = u2;
	vert.v = v1;
	ta->commit_vertex(&vert, sizeof(vert));
}

// Add an abitrary portion of a texture page as a centered sprite
void addPage(uint32 txr, int tx1, int ty1, int tx2, int ty2) {
	float x1, y1;

	x1=(640.0-(tx2-tx1))/2.0;
	y1=(480.0-(ty2-ty1))/2.0;

	addPage2(txr, x1, y1, tx1, ty1, tx2, ty2, 80);
}

// Add a specific portion of a texture page as a positioned sprite with alpha
void addPage2(uint32 txr, int x, int y, int tx1, int ty1, int tx2, int ty2, int alpha) {
	vertex_ot_t vert ;
	poly_hdr_t poly ;
	float u1, u2, v1, v2;
	float x1, x2, y1, y2;
	float alf;

	x1=x;
	x2=x1+(tx2-tx1)+1.0;
	y1=y;
	y2=y1+(ty2-ty1)+1.0;

	u1=tx1/256.0;
	u2=(tx2+1.0)/256.0;
	v1=ty1/256.0;
	v2=(ty2+1.0)/256.0;

	alf=(alpha/100.0);
	if (alf<0) alf=-alf;

	ta->poly_hdr_txr(&poly, TA_TRANSLUCENT, TA_ARGB1555, 256, 256, txr, TA_NO_FILTER);
	ta->commit_poly_hdr(&poly);

	vert.flags = TA_VERTEX_NORMAL;
	vert.x = x1;
	vert.y = y2;
	vert.z = 528.0f;
	vert.u = u1;
	vert.v = v2;
	vert.a = alf;
	vert.r = 1.0f;
	vert.g = 1.0f;
	vert.b = 1.0f;
	vert.oa = vert.or = vert.og = vert.ob = 0.0f;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = x1;
	vert.y = y1;
	vert.u = u1;
	vert.v = v1;
	ta->commit_vertex(&vert, sizeof(vert));
	
	vert.x = x2;
	vert.y = y2;
	vert.u = u2;
	vert.v = v2;
	ta->commit_vertex(&vert, sizeof(vert));

	vert.flags = TA_VERTEX_EOL;
	vert.x = x2;
	vert.y = y1;
	vert.u = u2;
	vert.v = v1;
	ta->commit_vertex(&vert, sizeof(vert));
}

